==> jungle-1.0.tar.gz: jungle-1.0/jungle/apps/timereport/report.py <==
# -*- coding: utf-8 -*-
from jungle.templatetags.jungle_tags import ugettext as _
from decimal import Decimal

CODICI = {
    2 : 2,
    4 : 13,
    5 : 18,
    6 : 22,
    7 : 30,
    9 : 21,
    10 : 12,
    12 : 14,
    13 : 19,
    14 : 28,
    15 : 27,
    16 : 26,
    17 : 31,
    19 : 1003,
    33 : 1001,
    34 : 32,
    36 : 32,
    }
def short_report(writer, T, Days, Users, Totals, PeriodTotals):
    writer.writerow([_('Employee'), _('Worked'), _('Extra'),
                    _('Holiday'), _('Illness')])
    for u in T.keys():
        user = Users[u]['user'] 
        writer.writerow([user.get_full_name(), 
                         num(PeriodTotals[u].worked),
                         num(PeriodTotals[u].extra),
                         num(PeriodTotals[u].holiday),
                         num(PeriodTotals[u].illness)] )
        

def long_report(writer, T, Days, Users, Totals, PeriodTotals):
    writer.writerow([_('Day'), _('Worked'), _('Extra'),
                    _('Holiday'), _('Illness')])
    for u in T.keys():
        user = Users[u]['user'] 
        writer.writerow([user.get_full_name()])
        for d in Days:
            t = Totals[u][d.day]
            writer.writerow([d,
                             num(Totals[u][d.day].worked),
                             num(Totals[u][d.day].extra ),
                             num(Totals[u][d.day].missing),
                             num(Totals[u][d.day].holiday ),
                             num(Totals[u][d.day].illness )])
                             
        writer.writerow([_('Total'), 
                         num(PeriodTotals[u].worked),
                         num(PeriodTotals[u].extra),
                         num(PeriodTotals[u].holiday),
                         num(PeriodTotals[u].illness)] )
        
def report_commercialista(writer, T, Days, Users, Totals, PeriodTotals):
    '''
    crea un report tipo:
    "1";"6";"28";"21";"100";"ROSSI";"MARIO";"01/10/2008";"8.00";"8.00";"LAV";"ORE LAVORATE";"8.00";""
    "1";"6";"28";"21";"100";"ROSSI";"MARIO";"02/10/2008";"8.00";"6.50";"LAV";"ORE LAVORATE";"6.50";""
    '''
    from copy import copy
    header = ['Consulente','Azienda','Filiale','Codice','Matricola','Cognome','Nome','Giorno',
              'Teoriche','Lavorate','Giustificativo','Descrizione','Ore','Tipologia']
    PRE = ['1','208','1',]
    writer.writerow(header)

    for u in T.keys():
        user = Users[u]['user'] 
        USER_DEF = [CODICI.get(user.id, 'MANCA'), '', user.last_name, user.first_name]
        
        for d in Days:
            t = Totals[u][d.day]

            ## ore lavorate
            profile = user.userprofile.time_profile
            teo = profile.get_day_hours(d.day)

            line = PRE + USER_DEF +[d.d(), teo, num2(t.worked), ]
            l = line + ['LAV', 'Ore Lavorate', num2(t.worked)]
            writer.writerow(l)

            ## Straordinario
            if t.extra:
                l = line + ['ST15', 'Straordinario', num2(t.extra)]
                writer.writerow(l)
                #printed = True

            ##  ferie
            if d.is_holiday():

                if teo:
                    if not t.worked:
                        if d.day.isoweekday() < 7:
                            GIUS = 'FESSE'
                            DESCR = 'festività infrasettimanali'
                            ore = teo
                        else:
                            GIUS = 'FER'
                            DESCR = 'Ferie godute'
                            ore = teo
                    
                        l = line + [GIUS, DESCR, ore]
                        writer.writerow(l)
                        #printed = True
                    else:
                        diff = teo - dec(t.worked)

                        if diff > 0 and d.day.isoweekday() < 7:
                            ## ha lavorato meno del teorico, esiste anche FESSE
                            l = line + ['FESSE', 'Feriale infrasettimanale', diff]
                            writer.writerow(l)
                            #printed = True
                            
            if t.holiday:
                l = line + ['FER', 'Ferie godute', num2(t.holiday)]
                writer.writerow(l)
                #printed = True
                
            ## malattia
            if t.illness:
                l = line + ['MAL', 'Malattia', num2(t.worked)]
                writer.writerow(l)
                #printed = True

def db_report(T, Days, Users, Totals, PeriodTotals):
    """
    Adds all data in timereport_final, that can be edited before generating the report
    """

    from jungle.timereport.models import Final

    q = Final.objects.filter(user__in=Users.keys())
    q = q.filter(date__gte=Days[0].day, date__lte=Days[-1].day)
    q.delete()

    for u in T.keys():
        user = Users[u]['user']

        for d in Days:
            t = Totals[u][d.day]
            profile = user.userprofile.time_profile
            teo = profile.get_day_hours(d.day)

            f = Final(
                status = 1,
                date = d.day,
                user_id = u,
                theoric = teo or None,
                use_extra = False,
                notes = ''
                )

            for field_name in ('worked', 'extra', 'missing', 'illness', 'holiday'):
                value = getattr(Totals[u][d.day], field_name)
                if value:
                    setattr(f, field_name, dec(value)) 

            ##  ferie
            if d.is_holiday():

                if teo:
                    if t.worked:
                        diff = teo - dec(t.worked)

                        if diff > 0 and d.day.isoweekday() < 7:
                            ## ha lavorato meno del teorico, esiste anche FESSE
                            f.fesse = diff

                    else:
                        if d.day.isoweekday() < 7:
                            f.fesse = teo
                        else:
                            f.holiday = teo
                    
            missing = teo - (f.worked or 0) - (f.illness or 0) - (f.holiday or 0) - (f.fesse or 0)
            if missing > 0:
                f.missing = missing

            f.save()
    msg = 'All values correctly inserted in table Final'
    return msg

def num(str):
    if not str:
        return ''
    h, m = str.split(':')
    if h == '0' and m == '0':
        return ''
    return "%s,%s" % (h, int(m)*25/15)

def num2(str):
    if not str:
        return '0.0'
    h, m = str.split(':')
    if h == '0' and m == '0':
        return '0.0'
    return "%s.%s" % (int(h), int(m)*25/15)

def dec(str):
    if not str:
        return 0
    h, m = str.split(':')
    if h == '0' and m == '0':
        return 0
    return Decimal("%s.%s" % (h, int(m)*25/15))

def minutes(str):
    if not str:
        return None
    h, m = str.split(':')
    return int(h)*60 + int(m)

def create_report(writer, T, Days, Users, Totals, PeriodTotals, mode=None):
    if mode == 'short':
        short_report(writer, T, Days, Users, Totals, PeriodTotals)
    elif mode == 'long':
        long_report(writer, T, Days, Users, Totals, PeriodTotals)
    elif mode == 'comm':
        return report_commercialista(writer, T, Days, Users, Totals, PeriodTotals)
    elif mode == 'db':
        return db_report(T, Days, Users, Totals, PeriodTotals)

==> jungle-1.0.2.tar.gz: jungle-1.0.2/jungle/apps/timereport/report.py <==
# -*- coding: utf-8 -*-
from jungle.templatetags.jungle_tags import ugettext as _
from decimal import Decimal

CODICI = {
    2: 2,
    4: 13,
    5: 18,
    6: 22,
    7: 30,
    9: 21,
    10: 12,
    12: 14,
    13: 19,
    14: 28,
    15: 27,
    16: 26,
    17: 31,
    19: 1003,
    33: 1001,
    34: 32,
    36: 32,
}
def short_report(writer, T, Days, Users, Totals, PeriodTotals):
    writer.writerow([_('Employee'), _('Worked'), _('Extra'),
                     _('Holiday'), _('Illness')])
    for u in T.keys():
        user = Users[u]['user']
        writer.writerow([user.get_full_name(),
                         num(PeriodTotals[u].worked),
                         num(PeriodTotals[u].extra),
                         num(PeriodTotals[u].holiday),
                         num(PeriodTotals[u].illness)])


def long_report(writer, T, Days, Users, Totals, PeriodTotals):
    writer.writerow([_('Day'), _('Worked'), _('Extra'),
                     _('Holiday'), _('Illness')])
    for u in T.keys():
        user = Users[u]['user']
        writer.writerow([user.get_full_name()])
        for d in Days:
            t = Totals[u][d.day]
            writer.writerow([d,
                             num(Totals[u][d.day].worked),
                             num(Totals[u][d.day].extra),
                             num(Totals[u][d.day].missing),
                             num(Totals[u][d.day].holiday),
                             num(Totals[u][d.day].illness)])

        writer.writerow([_('Total'),
                         num(PeriodTotals[u].worked),
                         num(PeriodTotals[u].extra),
                         num(PeriodTotals[u].holiday),
                         num(PeriodTotals[u].illness)])

def report_commercialista(writer, T, Days, Users, Totals, PeriodTotals):
    '''
    crea un report tipo:
    "1";"6";"28";"21";"100";"ROSSI";"MARIO";"01/10/2008";"8.00";"8.00";"LAV";"ORE LAVORATE";"8.00";""
    "1";"6";"28";"21";"100";"ROSSI";"MARIO";"02/10/2008";"8.00";"6.50";"LAV";"ORE LAVORATE";"6.50";""
    '''
    header = [
        'Consulente',
        'Azienda',
        'Filiale',
        'Codice',
        'Matricola',
        'Cognome',
        'Nome',
        'Giorno',
        'Teoriche',
        'Lavorate',
        'Giustificativo',
        'Descrizione',
        'Ore',
        'Tipologia']
    PRE = ['1', '208', '1', ]
    writer.writerow(header)

    for u in T.keys():
        user = Users[u]['user']
        USER_DEF = [CODICI.get(user.id, 'MANCA'), '', user.last_name, user.first_name]

        for d in Days:
            t = Totals[u][d.day]

            ## ore lavorate
            profile = user.userprofile.time_profile
            teo = profile.get_day_hours(d.day)

            line = PRE + USER_DEF + [d.d(), teo, num2(t.worked), ]
            l = line + ['LAV', 'Ore Lavorate', num2(t.worked)]
            writer.writerow(l)

            ## Straordinario
            if t.extra:
                l = line + ['ST15', 'Straordinario', num2(t.extra)]
                writer.writerow(l)
                #printed = True

            ##  ferie
            if d.is_holiday():

                if teo:
                    if not t.worked:
                        if d.day.isoweekday() < 7:
                            GIUS = 'FESSE'
                            DESCR = 'festività infrasettimanali'
                            ore = teo
                        else:
                            GIUS = 'FER'
                            DESCR = 'Ferie godute'
                            ore = teo

                        l = line + [GIUS, DESCR, ore]
                        writer.writerow(l)
                        #printed = True
                    else:
                        diff = teo - dec(t.worked)

                        if diff > 0 and d.day.isoweekday() < 7:
                            ## ha lavorato meno del teorico, esiste anche FESSE
                            l = line + ['FESSE', 'Feriale infrasettimanale', diff]
                            writer.writerow(l)
                            #printed = True

            if t.holiday:
                l = line + ['FER', 'Ferie godute', num2(t.holiday)]
                writer.writerow(l)
                #printed = True

            ## malattia
            if t.illness:
                l = line + ['MAL', 'Malattia', num2(t.worked)]
                writer.writerow(l)
                #printed = True

def db_report(T, Days, Users, Totals, PeriodTotals):
    """
    Adds all data in timereport_final, that can be edited before generating the report
    """

    from jungle.timereport.models import Final

    q = Final.objects.filter(user__in=Users.keys())
    q = q.filter(date__gte=Days[0].day, date__lte=Days[-1].day)
    q.delete()

    for u in T.keys():
        user = Users[u]['user']

        for d in Days:
            t = Totals[u][d.day]
            profile = user.userprofile.time_profile
            teo = profile.get_day_hours(d.day)

            f = Final(
                status=1,
                date=d.day,
                user_id=u,
                theoric=teo or None,
                use_extra=False,
                notes=''
            )

            for field_name in ('worked', 'extra', 'missing', 'illness', 'holiday', 'cassa'):
                value = getattr(Totals[u][d.day], field_name)
                if value:
                    setattr(f, field_name, dec(value))

            ##  ferie
            if d.is_holiday():

                if teo:
                    if t.worked:
                        diff = teo - dec(t.worked)

                        if diff > 0 and d.day.isoweekday() < 7:
                            ## ha lavorato meno del teorico, esiste anche FESSE
                            f.fesse = diff

                    else:
                        if d.day.isoweekday() < 7:
                            f.fesse = teo
                        else:
                            f.holiday = teo

            missing = teo - (f.worked or 0) - (f.illness or 0) - \
                (f.holiday or 0) - (f.fesse or 0) - (f.cassa or 0)
            if missing > 0:
                f.missing = missing

            f.save()
    msg = 'All values correctly inserted in table Final'
    return msg

def num(str):
    if not str:
        return ''
    h, m = str.split(':')
    if h == '0' and m == '0':
        return ''
    return "%s,%s" % (h, int(m)*25/15)

def num2(str):
    if not str:
        return '0.0'
    h, m = str.split(':')
    if h == '0' and m == '0':
        return '0.0'
    return "%s.%s" % (int(h), int(m)*25/15)

def dec(str):
    if not str:
        return 0
    h, m = str.split(':')
    if h == '0' and m == '0':
        return 0
    return Decimal("%s.%s" % (h, int(m)*25/15))

def minutes(str):
    if not str:
        return None
    h, m = str.split(':')
    return int(h)*60 + int(m)

def create_report(writer, T, Days, Users, Totals, PeriodTotals, mode=None):
    if mode == 'short':
        short_report(writer, T, Days, Users, Totals, PeriodTotals)
    elif mode == 'long':
        long_report(writer, T, Days, Users, Totals, PeriodTotals)
    elif mode == 'comm':
        return report_commercialista(writer, T, Days, Users, Totals, PeriodTotals)
    elif mode == 'db':
        return db_report(T, Days, Users, Totals, PeriodTotals)
